import { EditorSelection, Facet, Prec, SelectionRange, StateEffect, combineConfig, } from "@codemirror/state";
import { EditorView, RectangleMarker, layer, } from "@codemirror/view";
export const FakeSelection = StateEffect.define();
const fakeSelectionConfig = Facet.define({
    combine(configs) {
        return combineConfig(configs, {
            cursorBlinkRate: 1200,
            cursorStyle: "block",
            drawRangeCursor: true,
        }, {
            cursorBlinkRate: (a, b) => a ?? b,
            cursorStyle: (a, b) => a ?? b,
            drawRangeCursor: (a, b) => a ?? b,
        });
    },
});
/**
 * CodeMirror extension to imitate selections in replay.
 */
export function fakeSelection(config = {}) {
    return [
        fakeSelectionConfig.of(config),
        fakeCursorLayer,
        fakeSelectionLayer,
        style,
    ];
}
/// Retrieve the [`drawSelection`](#view.drawSelection) configuration
/// for this state. (Note that this will return a set of defaults even
/// if `drawSelection` isn't enabled.)
export function getDrawFakeSelectionConfig(state) {
    return state.facet(fakeSelectionConfig);
}
function configChanged(update) {
    return (update.startState.facet(fakeSelectionConfig) !==
        update.state.facet(fakeSelectionConfig));
}
const fakeCursorLayer = layer({
    above: true,
    markers(view) {
        const { state } = view;
        const conf = state.facet(fakeSelectionConfig);
        const cursors = [];
        if (!this.range)
            return [];
        for (const r of [this.range]) {
            if (r.empty || conf.drawRangeCursor) {
                const className = conf.cursorStyle === "block"
                    ? "lqv-fakeCursorBlock"
                    : "lqv-fakeCursorColumn";
                const cursor = r.empty
                    ? r
                    : EditorSelection.cursor(r.head, r.head > r.anchor ? -1 : 1);
                for (const piece of RectangleMarker.forRange(view, className, cursor))
                    cursors.push(piece);
            }
        }
        return cursors;
    },
    update(update, dom) {
        const effects = update.transactions
            .map((tr) => tr.effects.filter((e) => e.is(FakeSelection)))
            .reduce((a, b) => a.concat(b), []);
        const confChange = configChanged(update);
        if (confChange)
            setBlinkRate(update.state, dom);
        if (effects.length === 0) {
            return update.docChanged || update.selectionSet || confChange;
        }
        if (effects.length > 0) {
            dom.style.animationName =
                dom.style.animationName === "lqv-blink" ? "lqv-blink2" : "lqv-blink";
            this.range = SelectionRange.fromJSON(effects[effects.length - 1].value);
            return true;
        }
    },
    mount(dom, view) {
        setBlinkRate(view.state, dom);
    },
    class: "lqv-fakeCursorLayer",
});
function setBlinkRate(state, dom) {
    dom.style.animationDuration = `${state.facet(fakeSelectionConfig).cursorBlinkRate}ms`;
}
const fakeSelectionLayer = layer({
    above: false,
    markers(view) {
        if (!this.range) {
            return [];
        }
        return RectangleMarker.forRange(view, "lqv-fakeSelectionBackground", this.range);
    },
    update(update) {
        const effects = update.transactions
            .map((tr) => tr.effects.filter((e) => e.is(FakeSelection)))
            .reduce((a, b) => a.concat(b), []);
        if (effects.length === 0) {
            return (update.docChanged ||
                update.selectionSet ||
                update.viewportChanged ||
                configChanged(update));
        }
        if (effects.length > 0) {
            this.range = SelectionRange.fromJSON(effects[effects.length - 1].value);
            return true;
        }
    },
    class: "lqv-fakeSelectionLayer",
});
const themeSpec = {
    ".lqv-fakeSelectionBackground": {
        background: "#d7d4f0",
    },
    ".lqv-fakeCursorLayer": {
        animation: "steps(1) lqv-blink 1.2s infinite",
        pointerEvents: "none",
    },
    // Two animations defined so that we can switch between them to
    // restart the animation without forcing another style
    // recomputation.
    "@keyframes lqv-blink": { "0%": {}, "50%": { opacity: 0 }, "100%": {} },
    "@keyframes lqv-blink2": { "0%": {}, "50%": { opacity: 0 }, "100%": {} },
    ".lqv-fakeCursorBlock": {
        background: "#f00a",
        width: "1ch",
        marginLeft: "-0.6px",
        pointerEvents: "none",
    },
    ".lqv-fakeCursorColumn": {
        borderLeft: "1.2px solid black",
        marginLeft: "-0.6px",
        pointerEvents: "none",
    },
};
const style = Prec.highest(EditorView.theme(themeSpec));
