import { ChangeSet, type Text } from "@codemirror/state";
import type { EditorView } from "@codemirror/view";
import type { ReplayData } from "@liqvid/utils/replay-data";
import type { MediaElement } from "@lqv/playback";
import type { ScrollAction } from "./recording";
export { fakeSelection, FakeSelectionConfig } from "./fake-selection";
/** Possible replay commands. */
export type Action = ScrollAction | string | [changes: ChangeSet, selection?: [number, number]];
/** Reserved command for specifying file. */
export declare const selectCmd = "file:";
/** Reserved string for specifying scroll actions. */
export declare const scrollCmd = "s";
/** Key for the default view (in single-file mode). */
export declare const defaultViewName = "default";
/**
 * Replay typing in CodeMirror.
 * @returns Unsubscription function.
 */
export declare function cmReplay({ data, handle, playback, scrollBehavior, didScroll, shouldScroll, start, view, }: Omit<Parameters<typeof cmReplayMultiple>[0], "handle" | "shouldScroll" | "views"> & {
    /**
     * Function for handling special commands.
     * @param cmd Command to handle.
     * @param doc CodeMirror document.
     */
    handle?: (cmd: string, doc: Text) => void;
    /**
     * Callback that gets called when the replay scrolls to a new position.
     * You can use this to try to distinguish user scroll events from programmatic ones.
     */
    didScroll?: (scrollToOptions: ScrollToOptions) => void;
    /**
     * Callback for determining whether to replay scrolling. You can have this
     * return `false` to relinquish scrolling control back to the viewer.
     * By default, always returns `true`, i.e. scrolls are always replayed.
     */
    shouldScroll?: () => boolean;
    /** CodeMirror instance to sync with. */
    view: EditorView;
}): () => void;
/**
 * Replay typing to several CodeMirror instances in parallel.
 * @returns Unsubscription function.
 */
export declare function cmReplayMultiple({ data, didScroll, handle, playback, scrollBehavior, shouldScroll, start, views, }: {
    /** Recording data to replay. */
    data: ReplayData<Action>;
    /**
     * Function for handling special commands.
     * @param cmd Command to handle.
     * @param docs CodeMirror documents.
     */
    handle?: (cmd: string, docs: Record<string, Text>) => void;
    /**
     * Scroll behavior to pass to {@link Element.scrollTo}.
     * @default "auto"
     */
    scrollBehavior?: ScrollBehavior;
    /**
     * Callback that gets called when the replay scrolls to a new position.
     * You can use this to try to distinguish user scroll events from programmatic ones.
     */
    didScroll?: (filename: string, scrollToOptions: ScrollToOptions) => void;
    /**
     * Callback for determining whether to replay scrolling for a given file.
     * You can have this return `false` to relinquish scrolling control back
     * to the viewer.
     * By default, always returns `true`, i.e. scrolls are always replayed.
     */
    shouldScroll?: (filename: string) => boolean;
    /** Playback to sync with. */
    playback: MediaElement;
    /**
     * Time *in seconds* playback should start.
     * @default 0
     */
    start?: number;
    /** CodeMirror instances to sync with. */
    views: Record<string, EditorView>;
}): () => void;
